/*
   Name:    "ShootOut2-oo.rex"

   Invoke:  "java com.ibm.bsf.Main -lang rexx -in ShootOut2-oo.rex"
                or
            "java com.ibm.bsf.Main            -in ShootOut2-oo.rex"
                or
            "rexxj ShootOut2-oo.rex"
                or
            "rexx ShootOut2-oo.rex"

   Purpose:  create a window with a choice-object and two text-fields and buttons;
             if the ok-button is pressed then popup another window and display what
             was chosen and entered.

   Needs:    IBM's "Bean Scripting Framework" (BSF)
             (cf. "http://oss.software.ibm.com/developerworks/projects/bsf")

             and the package "bsf4rexx"
             (cf. "http://nestroy.wi-inf.uni-essen.de/Forschung/rgf/Entwicklung.html")

   Date:    2001-04-22 -- *NOT* finished yet!
            2003-01-23

   Author:   Rony G. Flatscher, WU Wien University,
*/

if rxFuncQuery("BSF") then  /* no bsf4rexx support available yet, hence load Java */
do
   call rxFuncAdd "BsfLoadFuncs", "BSF4Rexx", "BsfLoadFuncs"
   call BsfLoadFuncs    /* load all BSF-functions                    */
   call BsfLoadJava     /* load the JVM and set up BSF4Rexx-support  */
end

if      BsfInvokedBy()=1 then say "This Rexx program was invoked by Java!"
else if BsfInvokedBy()=2 then say "This Rexx program was invoked by Rexx, JVM loaded by Rexx!"
else                          say "No JVM present, we got troubles ..."

        /* import Java classes          */
.bsf~import("awtButton",        "java.awt.Button")
.bsf~import("awtFrame",         'java.awt.Frame')
.bsf~import("awtGBConstraints", 'java.awt.GridBagConstraints')
.bsf~import("awtGridBagLayout", 'java.awt.GridBagLayout')
.bsf~import("awtInsets",        'java.awt.Insets')
.bsf~import("awtLabel",         "java.awt.Label")
.bsf~import("awtTextArea",      "java.awt.TextArea")
.bsf~import("awtTextField",     "java.awt.TextField")

.bsf~import("awtDialog",        "java.awt.Dialog")
.bsf~import("awtMenuBar",       "java.awt.MenuBar" )
.bsf~import("awtMenu",          "java.awt.Menu")
.bsf~import("awtMenuItem",      "java.awt.MenuItem")


        /* create labels                */
label1 = .awtLabel~new('String', 'Your question?')
label2 = .awtLabel~new('St'    , 'Well, consider this answer:' )

        /* get static values from class, save in .local environment     */
.local~scr.None   = .bsf~getStaticValue('java.awt.TextArea', 'SCROLLBARS_NONE')
.local~scr.Vert   = .bsf~getStaticValue('java.awt.TextArea', 'SCROLLBARS_VERTICAL_ONLY')

        /* create text area objects     */
taQuery  = .awtTextArea~new('str', "", 'i',   5, 'i', 80, 'i', .scr.None)
taAnswer = .awtTextArea~new('str', "" , 'i', 15, 'i', 80, 'i', .scr.Vert)

        /* make the answer text area non-editable       */
taAnswer~setEditable("boolean", .false)

        /* create the buttons           */
butAnswer = .awtButton~new('String', ' Please answer me! ')
butLeave  = .awtButton~new('String', ' Let us stop for dooday, o.k.? ' )

        /* create gridBagLayout & constraint    */
lay    = .awtGridBagLayout~new
layCon = .awtGBConstraints~new

        /* now define details of the GridBagLayout      */
layCon~bsf.setFieldValue( 'fill',       'i', bsf('getFieldValue', layCon, 'HORIZONTAL'))
layCon~bsf.setFieldValue( 'gridwidth',  'i', bsf('getFieldValue', layCon, 'REMAINDER' ))

inset = .awtInsets~new('int', 5, 'int', 5, 'int', 5, 'int', 5)
layCon~bsf.setFieldValue( 'insets',     'o', inset)
.local~lay.con=layCon   -- save for later use in the local environment under the name "LAY.CON"

        /* register the constraints with the layout manager     */
lay ~~setConstraints('o', label1,    'o', layCon)
lay ~~setConstraints('o', taQuery,   'o', layCon)
lay ~~setConstraints('o', butAnswer, 'o', layCon)
lay ~~setConstraints('o', label2,    'o', layCon)
lay ~~setConstraints('o', taAnswer,  'o', layCon)
lay ~~setConstraints('o', butLeave,  'o', layCon)


        /* create and assemble menu     */
mb    = .awtMenuBar~new

m     = .awtMenu     ~new("string", "File")
medit = .awtMenuItem ~new("string", "Edit answers...")
mexit = .awtMenuItem ~new("string", "Exit")
m ~~add("o", medit) ~~add("o", mexit)   -- add menu items to menu

mb ~add("o", m)                         -- add menu to menu bar

        /* create the frame window                              */
window = .awtFrame~new('String', 'ShootOut!')
window~~setMenuBar("o", mb)

        /* now add controls to window, change some settings of it       */
window ~~setLayout('Object', lay)
window ~~add('O', label1) ~~add('O', taQuery)  ~~add('O', butAnswer)
window ~~add('O', Label2) ~~add('O', taAnswer) ~~add('O', butLeave)

        /* pack the elements, show the window, bring it to the front    */
window ~~pack ~~show ~~toFront

        /* define the events we are interested in and the text to be sent to us */
window    ~ bsf.addEventListener('window', 'windowClosing', 'call BSF "exit"')

butAnswer ~ bsf.addEventListener('action', '', 'call answer taQuery, taAnswer')
butLeave  ~ bsf.addEventListener('action', '', 'call BSF "exit"')

medit~bsf.addEventListener("action", "", "call pwd_popup 'show', window")
mexit~bsf.addEventListener("action", "", ".bsf~exit")

.local~file.name="ShootAnswers.txt"             /* save file name explicitly    */

        /* read answers from file       */
call read_answers

do forever      /* wait and fetch the text to be sent by Java           */
   a = bsf("pollEventText")     /* wait for an eventText to be sent     */
   say "... ["a"] ..."

   interpret a                  /* execute as a Rexx program    */
   if result= "SHUTDOWN, REXX !" then leave     /* JVM will be shutdown */
end
exit

::requires "BSF.cls"    -- "requires" must be the first of the directives,
                        -- this one is needed for getting access to the Object Rexx
                        -- BSF wrapper classes


::routine read_answers
   s=.stream~new(.file.name)~~open("read")
   .local~answers=s~arrayin
   s~close

        /* react to pressing the answer button  */
::routine answer
   use  arg taQuery, taAnswer           /* get names of TextAreas                                       */
   txt=taQuery~getText                  /* get text             */

   lf = "0a"x
   txt=txt~translate("  ", "0d0a"x)~space~strip('T')
   if txt~right(1) = "?" then           /* a question in hand?          */
   do
      newLine = "0a0a"x
      tmp     =        newLine || "*** your question: ***" || newLine || txt
      tmp     = tmp || newLine || "--- my answer to you! ---" || newLine
      pick    = random(1, .answers~items)/* pick a number        */
      tmp     = tmp || .answers~at(pick)

      call bsf 'invoke', taAnswer, 'append',  'Str', tmp /* append to answer text area   */
      call bsf 'invoke', taQuery,  'setText', 'Str', ""
   end
   return


   /* Password Popup            */
::routine pwd_popup
   parse arg action

   if action="show" then        /* set up and show dialog       */
   do
       -- create an invisible window
       if \.local~hasentry("pop.win") then
       do
           use arg _, window    -- get owner of this (modal) dialog
           .local~pop.win    = .awtDialog~new("obj", window, "Str", "Password check!")
                -- define a grid layout for this dialog
           .pop.win~setLayout("obj", .bsf~new("java.awt.GridLayout", .array~of("int", 2, "int", 2) ))

           .local~pop.tf     = .awtTextField~new

           .pop.tf~~setEchoChar("Char", "*")

           pop.ok = .awtButton~new("str", "O.K.!")
           pop.no = .awtButton~new("str", "Abort")

                -- add the components
           .pop.win ~~add("obj", .awtLabel~new("str", "Password? ('hey')")) ~~add("obj", .pop.tf)
           .pop.win ~~add("obj", pop.ok) ~~add("obj", pop.no)

               -- define eventListeners
           pop.ok   ~~bsf.addEventListener("action", ""            ,  "call pwd_popup 'button', 'ok'")
           pop.no   ~~bsf.addEventListener("action", ""            ,  "call pwd_popup 'button', 'no'")
           .pop.win ~~bsf.addEventListener("window", "windowClosing", "call pwd_popup 'button', 'no'")
           .pop.win ~~setResizable("boolean", .false) ~~pack
       end
       -- make dialog visible
--       .pop.win ~~show ~~setModal("bool", .true)
       .pop.win ~~show
   end
   else if action="button" then         -- button was pressed
   do
      if arg(2)="no" then       -- Abort button was pressed, just hide
      do
--         .pop.win ~~setModal("bool", .false) ~~hide          -- hide dialog
         .pop.win ~~hide          -- hide dialog

      end
      else
      do
         txt= .pop.tf~getText   -- get text
         if txt \= "hey" then   -- wrong password, make entry visible
         do
            call beep 2500, 200
--            .pop.tf~setEchoChar("char", "0"x)
            .pop.tf~setText("str", .pop.tf~getText)
--            .pop.win ~~show ~~setModal("bool", .true)
            .pop.win ~~show
         end
         else
         do     -- add the next eventText to the eventList
            .bsf~postEventText("call edit_popup 'show', window, answers")
--            .pop.win ~~setModal("bool", .false) ~~hide       -- hide dialog
            .pop.win ~~hide       -- hide dialog
         end
      end
   end
   return



   /* Popup window for editing answers. */
::routine edit_popup
   parse arg action

   if action="show"  then       /* set up and show dialog       */
   do
       -- create an invisible window
       if \.local~hasentry("pop_edit.win") then
       do
           use arg _, window          -- get owner of this (modal) dialog
           .local~pop_edit.win    = .awtDialog~new("obj", window, "Str", "Edit Answers!")

           .local~pop_edit.ta = .awtTextArea~new('str', "" , 'i', 27, 'i', 80, 'i', .scr.Vert)
           .local~pop_edit.tf     = .awtTextField~new
           pop_edit.ok = .awtButton~new("str", "O.K., save and use my changes!")
           pop_edit.no = .awtButton~new("str", "Abort, please just return, will you?")

                -- define the layout
           lay    = .awtGridBagLayout~new
           .pop_edit.win ~~setLayout('Object', lay)

           lay ~~setConstraints('o', .pop_edit.ta,    'o', .lay.Con)
           lay ~~setConstraints('o',  pop_edit.ok,    'o', .lay.Con)
           lay ~~setConstraints('o',  pop_edit.no,    'o', .lay.Con)

                -- add the components
           .pop_edit.win ~~add("obj", .pop_edit.ta) ~~add("obj",  pop_edit.ok) ~~add("obj",  pop_edit.no)

               -- define eventListeners
           pop_edit.ok   ~~bsf.addEventListener("action", "", "call edit_popup 'button', 'ok'")
           pop_edit.no   ~~bsf.addEventListener("action", "", "call edit_popup 'button', 'no'")
           .pop_edit.win ~~bsf.addEventListener("window", "windowClosing", "call edit_popup 'button', 'no'")
           .pop_edit.win ~~pack

               -- set answers text
          txt=charin(.file.name, 1, stream(.file.name, "C", "QUERY SIZE"))

--          .pop_edit.ta~setText("str", txt)
          call bsf "invoke", .pop_edit.ta, "setText", "str", txt

-- answers = .stream~new(.file.name )~arrayin      /* read answers from file       */

       end
       -- make dialog visible
       .pop_edit.win ~~setResizable("boolean", .false) ~~show
--       .pop_edit.win ~~setModal("boo", .true)
   end
   else if action="button" then         -- button was pressed
   do
      if arg(2)="no" then               -- Abort button was pressed, just hide
      do
--         .pop_edit.win ~~setModal("boo", .false)
         .pop_edit.win~hide             -- hide dialog
      end
      else
      do
         call stream .file.name, "c", "open write replace"
--         s=.stream~new(.file.name)
--         say "s~~open(wr):" s~~open("write replace")
         txt= .pop_edit.ta~getText      -- get text
         do while txt <> ""
            parse var txt before "." txt
            before=before~translate("  ", "0d0a"x)~space~strip('T')   -- remove empty lines and spaces
            if before = "" then iterate
--            s~lineout(strip(txt) || ".")
            call lineout .file.name, before || "."
say "writing: ["before||".]"
         end
         call stream .file.name, "C", "close"
--         s~close

--         .stream~new(.file.name) ~~open("write replace") ~~charout(txt) ~~close -- write text to file
say "just wrote something to" .file.name", now re-reading ..."

         call read_answers
--         .pop_edit.win ~~setModal("boo", .false)
         .pop_edit.win~hide             -- hide dialog
      end
   end
say ".pop_edit.win~isModal:" .pop_edit.win~isModal

   return



